
import(methods)
import(stats)
import(stabs)

import(Matrix)
import(parallel)
importFrom(survival, Surv, survfit)
importFrom(splines, bs, splineDesign)
importFrom(lattice, levelplot, barchart)
importFrom(nnls, nnls)
importFrom(quadprog, solve.QP)
importFrom(utils, packageDescription)
importFrom("graphics", "abline", "axis", "box", "lines", "matlines", "matplot", "plot", "points", "polygon")
importFrom("grDevices", "hcl", "rgb")
importFrom("partykit", "ctree_control", "extree_data", "extree_fit", "fitted_node")

export(glmboost,
       gamboost,
       blackboost,
       mboost,
       boost_control, mstop, Family,
       GaussReg, Gaussian, GaussClass, Laplace, Binomial, Poisson, GammaReg, QuantReg,
       ExpectReg, NBinomial, PropOdds, Weibull, Loglog, Lognormal, AUC, mboost_fit,
       Huber, AdaExp, Gehan, Cindex, CoxPH, Hurdle, Multinomial, RCG, FP, IPCweights,
       cvrisk, cv, 
       bbs, bols, bspatial, brandom, btree, bss, bns, brad, bkernel, bmono, bmrf, buser,
       survFit, selected, selected.mboost,
       nuisance, "%+%", "%X%", "%O%", extract, risk, "mstop<-",
       fitted.mboost, plot.glmboost, plot.mboost, lines.mboost, plot.cvrisk,
       predict.mboost, predict.glmboost,
       stabsel.mboost, stabsel_parameters.mboost,
       confint.mboost, confint.glmboost, plot.mboost.ci, lines.mboost.ci, print.glmboost.ci,
       downstream.test,
       varimp, plot.varimp, as.data.frame.varimp,
       mboost_intern)
       ###, basesel, fitsel)
exportClasses("boost_family")
exportMethods("show")

S3method(print, glmboost)
S3method(plot, glmboost)
S3method(glmboost, formula)
S3method(glmboost, matrix)
S3method(glmboost, default)
S3method(print, mboost)
S3method(summary, mboost)
S3method(print, summary.mboost)
S3method(plot, mboost)
S3method(lines, mboost)
S3method(plot, survFit)
S3method(coef, mboost)
S3method(coef, glmboost)
S3method(hatvalues, mboost)
S3method(hatvalues, gamboost)
S3method(hatvalues, glmboost)
S3method(fitted, mboost)
S3method(predict, mboost)
S3method(predict, glmboost)
S3method(AIC, mboost)
S3method(logLik, mboost)
S3method(resid, mboost)
S3method(variable.names, mboost)
S3method(variable.names, glmboost)
S3method(print, mboost)
S3method(print, cvrisk)
S3method(print, gbAIC)
S3method(plot, gbAIC)
S3method(plot, cvrisk)
S3method(mstop, gbAIC)
S3method(mstop, cvrisk)
S3method(mstop, mboost)
S3method(nuisance, mboost)
S3method(survFit, mboost)
S3method("[", mboost)
S3method(dpp, blg)
S3method(model.frame, blg)
S3method(model.frame, mboost)
S3method(fit, bl)
S3method(coef, bm_lin)
S3method(coef, bm_cwlin)
S3method(selected, mboost)
S3method(cvrisk, mboost)
# S3method(selected, glmboost)
S3method(update, mboost)
S3method(extract, mboost)
S3method(extract, glmboost)
S3method(extract, blackboost)
S3method(extract, blg)
S3method(extract, bl_lin)
S3method(extract, bl_tree)
S3method(residuals, mboost)
S3method(risk, mboost)
S3method(confint, mboost)
S3method(confint, glmboost)
S3method(plot, mboost.ci)
S3method(lines, mboost.ci)
S3method(print, glmboost.ci)
S3method(stabsel, mboost)
S3method(stabsel_parameters, mboost)
S3method(varimp, mboost)
S3method(plot, varimp)
S3method(as.data.frame, varimp)

useDynLib(mboost)
