
library("mboost")
set.seed(290875)


data("bodyfat", package = "TH.data")

object <- glmboost(DEXfat ~ ., data = bodyfat, center = TRUE)
a <- stabsel(object, cutoff = 0.9, q = 2)
a
plot(a)

object <- gamboost(DEXfat ~ ., data = bodyfat)
a <- stabsel(object, cutoff = 0.9, q = 2)
a
plot(a)
